
# q-expansion of newform 120.6.a.g, downloaded from the LMFDB on 22 January 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 120
weight = 6
poly_data = [-372, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[-4, 8], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0], [9, 0], [-25, 0], [8, 1], [32, -4], [582, 3], [1082, -5], [1320, -7], [20, 31], [970, -32], [3164, -13], [1766, 35], [1202, 106], [15964, -40], [19116, -23], [-12286, -38], [2592, 36], [-26978, -182], [47580, 88], [-35776, -208], [-48118, -26], [14644, -171], [-71508, 192], [-69862, -42], [-18430, 336], [-25006, 124], [-70264, -695], [2092, -342], [6214, -688], [-152814, 351], [168048, 1237], [-7944, -370], [73346, -751], [169808, -505], [34154, -400], [305372, -709], [-164386, -2483], [-135012, 1792], [51156, 1559], [-86686, 416], [-238880, 1106], [291710, -192], [270424, 428], [-271486, -4486], [158146, 4218], [401628, 579], [-644184, 2247], [-674360, 3751], [-8428, -3160], [-851906, 1594], [288090, 5365], [60240, -7758], [156930, 304], [-382944, -58], [842578, -2753], [568188, 3957], [-1470198, 3504], [362548, -9385], [-1242818, -579], [1446234, -1514], [393204, -2356], [408066, -11782], [991644, 8174], [-115304, 7290], [-326966, -4170], [-1609398, -4058], [-1570536, -3577], [-314046, 5942], [989412, 1698], [372462, 3494], [1814690, 6401], [-1371368, -14788], [1455352, 12135], [781734, 16873], [418792, -4671], [594796, 19217], [-2834454, -9024], [-3319714, -16041], [929722, 25984], [2004378, -23272], [-1470392, -25438], [1197118, -33494], [-4412720, 1956], [-1111102, 18694], [3153964, 18303], [1251884, 23768], [-4072214, -460], [-237222, 49400], [-4696542, -332], [-4014648, -18847], [2095252, 24786], [-6572120, 16192], [-37288, -23257], [-5662376, 18452], [755136, 8563], [-2712748, 7061], [-3515510, -5072], [-5591614, 4780], [-3935156, 5170], [1697926, 62528], [2104820, 68572], [1250354, 30220], [6637276, -35234], [1546178, -9524], [-11680072, 14299], [-949326, -24556], [4265180, -17362], [1942802, 36427], [594456, 77404], [-3223990, -20064], [7425776, -41953], [-7523754, 14791], [-1672406, 41709], [-1154656, -73329], [-4677492, 29909], [-10432278, 23742], [-9226516, -6196], [1679580, 38859], [8343074, -68704], [7155248, 1572], [284774, -35882], [-19642542, -13742], [8748842, -20032], [-2086436, -10730], [14137360, -39357], [8683994, -15764], [7419966, 20582], [8748168, -17250], [4177416, -50309], [-10694250, -49223], [-7642560, 33383], [-20920084, -48877], [14440108, -19037], [5732078, 92465], [-852126, 170932], [-4217454, 100668], [12444770, 73246], [-11239372, -35230], [22407954, 51464], [16454130, -35682], [-12924464, 2959], [3566130, -36860], [-13297616, -54475], [3139196, 31228], [282294, -141392], [-9155272, -121776], [11331990, 99995], [16713762, 149155], [9769112, 118853], [-21338324, 99111], [28838686, 41613], [-10480062, -12494], [-2644916, -63564], [-29888508, -36117], [5073268, -202842], [9000872, 93682], [-23174172, -67621], [7305402, -17856], [-6351302, -276688], [-33807054, -16420], [6218828, -135338], [3108802, -211343], [-9812768, 87371], [10084160, 130378], [212490, 219995], [-47482132, 58739], [8235708, -291659], [3908982, -57819], [16658178, -55472], [25347642, 39264], [-29901024, -22240], [-18081378, -231776], [-30894392, -31804], [8489658, -22714], [6983476, 108017], [51166058, -71326], [-22658200, 185981], [-58155062, 42916], [21445336, -242225], [-23743266, -33200], [30676008, -113515], [47774256, 13516], [46350046, -204591], [-5577462, -369921], [-22016324, -11939], [-64374566, -91036], [-56642058, 199], [54133364, -60930], [-1455654, 52472], [19552808, -63064], [-5120030, 58674], [2476404, 294582], [6449624, -181217], [-33913934, 128848], [4816684, 162140], [49096106, 217911], [-5526894, -305320], [20191158, -415381], [-52027918, 254915], [55327820, -201893], [-2443246, -527388], [48975236, -4599], [-2230370, 444639], [44930722, -361204], [53446792, 23834], [2957114, 311726], [5490780, 608863], [-8394196, 202954], [66796938, 281790], [-52219568, -154599], [39307506, -128594], [58165690, 169736], [43929528, -207895], [70188668, -198408], [-33241400, 560954], [76808618, 184420], [-54601320, -243205], [-1399670, -558036], [12410084, 59641], [35760642, 15876], [21419486, 82242], [-100757028, -170441], [29716138, 242110], [-23229576, 680857], [26887876, 350802], [-9279898, 6244], [6195402, 17151], [-33280632, -42690], [-31547312, -109275], [-65027064, 79998], [-15010186, 731415], [-28107832, 200055], [-69252556, 58503], [-5291958, -120886], [53545756, -12638], [65028588, 234339], [98788818, -177784], [-20119606, 309308], [18377944, 366738], [24246768, -105304], [-6976132, -477022], [74937480, -197609], [-8175824, -44831], [-98720722, -257166], [-12227774, -143319], [90237720, -153690], [84509128, -96597], [16526712, -361596], [-28962688, 871445], [7175244, 274675], [67896326, -205379], [-60702030, 352740], [-80229652, -429755], [-20090502, -225104], [-88945766, -128130], [113899200, -342024], [183996798, -65260], [92357436, -26926], [7088722, 250258], [-79011558, -31742], [-84855512, -510273], [-32329844, -596278], [-13114898, 1228770], [47839870, -172213], [-89676422, -301051], [113885200, 656283], [-79176350, -196636], [-11150182, -713876], [9816780, 1112762], [86384266, -59272], [139998526, 20732], [73194044, 86582], [-173091926, -231006], [-7132796, -1296747], [53552882, 640202], [78194824, -616499], [-174145212, -380370], [-203918642, 26362], [-174779734, 560312], [71614632, 120402], [-134621244, 746637], [4811652, 661157], [112229468, -100973], [-60623658, 273620], [56482756, -651642], [-63669800, 713368], [-4100766, -772452], [-33950918, -980200], [27021092, 802801], [-22764286, -461056], [-23177966, 1437960], [31404364, -581636], [-98754102, -265443], [-93262936, 175642], [7141998, 868127], [11725570, 1543688], [-16161308, -249411], [135825810, -953358], [120712920, 558054], [-26219052, 209428], [-63769670, 672342], [153090994, 584564], [107488820, 144295], [57195108, 644706], [-32929072, 1217393], [185900754, 358384], [-26767932, 290164], [1973350, 85918], [-40886728, -161738], [62683286, -550647], [203985436, -383689], [77986738, -1008704], [-20937094, -1078956], [79012924, -981582], [89551860, -1553939], [82944938, 1039300], [78387008, -786926], [-204491680, 219494], [66680018, -658198], [241552114, -99716], [-12010088, 1353569], [-209022870, 308046], [140340202, -180260], [973168, -982457], [147520466, 1468411], [-7930078, 554408], [88197384, 1030679], [-78150900, -288008], [-86553244, 980951], [58734682, -1779436], [-364660506, 257402], [155505778, -1063240], [-373638564, 53219], [212596420, 896822], [12471352, 1163913], [-200112284, -1571176], [124823374, 1775992], [-70037502, -829251], [-248136454, 781756], [-149077056, -546591], [-20886282, -541663], [114037570, -122477], [-222389758, 887848], [45766372, 396605], [-303882310, -976358], [-293196344, -233448], [-86186866, -239358], [115817660, 192376], [-20822560, -9410], [-4380974, 3170770], [104726944, -1587559], [99458970, -1631536], [53048554, -104636], [-195674680, 1212603], [-79224522, 1319506], [193932826, 347145], [70888416, -114036], [-114923672, -2532082], [-103025990, 715257], [50786364, 544601], [-155920314, 450841], [134615562, -1645920], [-5867340, -1590441], [89601032, 1954794], [-151746468, -40480], [-98101238, -1551760], [195152938, -2500430], [-40197072, 2123395], [286832746, -1651132], [226645888, -301370], [254730728, 103869], [-55710356, -2692587], [-96711038, 759288], [285071580, 1139503], [152473878, 441305], [-196230936, 1304448], [-373955400, 1314102], [174390946, -1226516], [159136554, 1585536], [139511930, 2847168], [-154171310, -1470716], [175047522, -2277991], [125287816, -1838039], [110360818, 765121], [-324116344, 1707267], [438815612, -778359], [27201020, 1257867], [143248030, 2082061], [-368208156, -619090], [2698508, 540747], [275426722, 2916088], [-187765830, -309844], [519279424, 97152], [193645156, 1488042], [-18135560, -1160904], [216294698, 693320], [243219964, -932295], [136882402, 609146], [203021152, -310255], [313349978, -1417496], [304461886, -520152], [-138688774, 2782481], [175804064, 1566653], [-48097502, -402715], [-267143022, 78384], [13576668, 1689413], [-309474794, 2509281], [-365279430, -112992], [-237104196, 682328], [19717496, -3209302], [6595874, 1901480], [-36200438, -947588], [-613473524, -178386], [-243116288, -846873], [-4903574, 294638], [159672922, 433260], [7187232, -3466656], [6986280, -4084319], [415099498, -2079599], [56659348, 697351], [-99083710, 3125760], [-218148538, -389615], [-224966636, -441245], [-151086968, 4327256], [48468634, -4211022], [-195260838, 4763754], [391657644, 1921112], [21094762, -3256484], [515556720, 777389], [-34228360, -265996]]
